@@ -62,7 +62,7 @@ class CommandArguments:
     use_buck_builder: Optional[bool] = None
     use_buck_source_database: Optional[bool] = None
     source_directories: List[str] = field(default_factory=list)
-    filter_directory: Optional[str] = None
+    do_not_ignore_errors_in: List[str] = field(default_factory=list)
     buck_mode: Optional[str] = None
     no_saved_state: bool = False
     search_path: List[str] = field(default_factory=list)
@@ -83,6 +83,7 @@ class CommandArguments:
     shared_memory_dependency_table_power: Optional[int] = None
     shared_memory_hash_table_power: Optional[int] = None
     number_of_workers: Optional[int] = None
+    enable_hover: Optional[bool] = None
 
 
 @dataclass(frozen=True)
@@ -140,6 +141,7 @@ class InferArguments:
     logging_sections: Optional[str] = None
     use_future_annotations: bool = False
     in_place: bool = False
+    simple_annotations: bool = False
     paths_to_modify: Optional[Set[Path]] = None
     print_only: bool = False
     read_stdin: bool = False
