@@ -8,8 +8,6 @@ import logging
 import os
 import shutil
 import sys
-import time
-import traceback
 from dataclasses import replace
 from pathlib import Path
 from typing import Iterable, List, Optional
@@ -17,50 +15,19 @@ from typing import Iterable, List, Optional
 import click
 
 from . import (
-    buck,
     command_arguments,
-    commands,
     configuration as configuration_module,
     filesystem,
     log,
     recently_used_configurations,
-    statistics_logger,
 )
-from .commands import Command, ExitCode, v2
-from .exceptions import EnvironmentException
+from . import commands
 from .version import __version__
 
 
 LOG: logging.Logger = logging.getLogger(__name__)
 
 
-def _log_statistics(
-    command: Command,
-    start_time: float,
-    client_exception_message: str,
-    error_message: Optional[str],
-    exit_code: int,
-) -> None:
-    configuration = command.configuration
-    if configuration is not None:
-        statistics_logger.log_with_configuration(
-            category=statistics_logger.LoggerCategory.USAGE,
-            configuration=configuration,
-            integers={
-                "exit_code": exit_code,
-                "runtime": int((time.time() - start_time) * 1000),
-            },
-            normals={
-                "cwd": os.getcwd(),
-                "client_version": __version__,
-                "command_line": " ".join(sys.argv),
-                "command": command.NAME,
-                "client_exception": client_exception_message,
-                "error_message": error_message,
-            },
-        )
-
-
 def _show_pyre_version_as_text(
     binary_version: Optional[str], client_version: str
 ) -> None:
@@ -99,59 +66,9 @@ def _start_logging_to_directory(log_directory: str) -> None:
     log.enable_file_logging(log_directory_path / "pyre.stderr")
 
 
-def run_pyre_command(
-    command: Command,
-    configuration: configuration_module.Configuration,
-    noninteractive: bool,
-) -> ExitCode:
-    start_time = time.time()
-
-    client_exception_message = ""
-    # Having this as a fails-by-default helps flag unexpected exit
-    # from exception flows.
-    exit_code = ExitCode.FAILURE
-    try:
-        _check_configuration(configuration)
-        _start_logging_to_directory(configuration.log_directory)
-        LOG.debug(f"Running cli command `{' '.join(sys.argv)}`...")
-        exit_code = command.run().exit_code()
-    except (buck.BuckException, EnvironmentException) as error:
-        client_exception_message = str(error)
-        exit_code = ExitCode.FAILURE
-        if isinstance(error, buck.BuckUserError):
-            exit_code = ExitCode.BUCK_USER_ERROR
-        elif isinstance(error, buck.BuckException):
-            exit_code = ExitCode.BUCK_INTERNAL_ERROR
-    except (configuration_module.InvalidConfiguration) as error:
-        client_exception_message = str(error)
-        exit_code = ExitCode.CONFIGURATION_ERROR
-    except commands.ClientException as error:
-        client_exception_message = str(error)
-        exit_code = error.exit_code
-    except Exception:
-        client_exception_message = traceback.format_exc()
-        exit_code = ExitCode.FAILURE
-    except KeyboardInterrupt:
-        LOG.warning("Interrupted by user")
-        LOG.debug(traceback.format_exc())
-        exit_code = ExitCode.SUCCESS
-    finally:
-        if len(client_exception_message) > 0:
-            LOG.error(client_exception_message)
-        result = command.result()
-        error_message = result.error if result else None
-        command.cleanup()
-        _log_statistics(
-            command,
-            start_time,
-            client_exception_message,
-            error_message,
-            exit_code,
-        )
-    return exit_code
-
-
-def _run_check_command(arguments: command_arguments.CommandArguments) -> ExitCode:
+def _run_check_command(
+    arguments: command_arguments.CommandArguments,
+) -> commands.ExitCode:
     configuration = _create_configuration_with_retry(arguments, Path("."))
     _check_configuration(configuration)
     _start_logging_to_directory(configuration.log_directory)
@@ -166,7 +83,7 @@ def _run_check_command(arguments: command_arguments.CommandArguments) -> ExitCod
         sequential=arguments.sequential,
         show_error_traces=arguments.show_error_traces,
     )
-    return v2.check.run(configuration, check_arguments)
+    return commands.check.run(configuration, check_arguments)
 
 
 def _run_incremental_command(
@@ -175,7 +92,7 @@ def _run_incremental_command(
     incremental_style: command_arguments.IncrementalStyle,
     no_start_server: bool,
     no_watchman: bool,
-) -> ExitCode:
+) -> commands.ExitCode:
     configuration = _create_configuration_with_retry(arguments, Path("."))
     _check_configuration(configuration)
     _start_logging_to_directory(configuration.log_directory)
@@ -198,7 +115,7 @@ def _run_incremental_command(
         terminal=False,
         wait_on_initialization=True,
     )
-    return v2.incremental.run(
+    return commands.incremental.run(
         configuration,
         command_arguments.IncrementalArguments(
             output=arguments.output,
@@ -208,7 +125,9 @@ def _run_incremental_command(
     )
 
 
-def _run_default_command(arguments: command_arguments.CommandArguments) -> ExitCode:
+def _run_default_command(
+    arguments: command_arguments.CommandArguments,
+) -> commands.ExitCode:
     if shutil.which("watchman"):
         return _run_incremental_command(
             arguments=arguments,
@@ -367,7 +286,12 @@ def _check_configuration(configuration: configuration_module.Configuration) -> N
         "Can be specified multiple times to include multiple directories."
     ),
 )
-@click.option("--filter-directory", type=str, hidden=True)
+@click.option(
+    "--do-not-ignore-errors-in",
+    type=str,
+    multiple=True,
+    help=("Report type errors within the given directory."),
+)
 @click.option(
     "--no-saved-state",
     is_flag=True,
@@ -436,6 +360,13 @@ def _check_configuration(configuration: configuration_module.Configuration) -> N
     help="Power of the hash table in shared memory.",
     hidden=True,
 )
+@click.option(
+    "--enable-hover/--no-enable-hover",
+    is_flag=True,
+    help="Whether Pyre should show types on hover in the IDE.",
+    default=None,
+    hidden=True,
+)
 @click.option("--number-of-workers", type=int, help="Number of parallel workers to use")
 def pyre(
     context: click.Context,
@@ -459,7 +390,7 @@ def pyre(
     buck_mode: Optional[str],
     use_buck_source_database: Optional[bool],
     source_directory: Iterable[str],
-    filter_directory: Optional[str],
+    do_not_ignore_errors_in: Iterable[str],
     no_saved_state: bool,
     search_path: Iterable[str],
     binary: Optional[str],
@@ -478,6 +409,7 @@ def pyre(
     shared_memory_dependency_table_power: Optional[int],
     shared_memory_hash_table_power: Optional[int],
     number_of_workers: Optional[int],
+    enable_hover: Optional[bool],
 ) -> int:
     arguments = command_arguments.CommandArguments(
         local_configuration=local_configuration,
@@ -498,7 +430,7 @@ def pyre(
         use_buck_builder=use_buck_builder,
         use_buck_source_database=use_buck_source_database,
         source_directories=list(source_directory),
-        filter_directory=filter_directory,
+        do_not_ignore_errors_in=list(do_not_ignore_errors_in),
         buck_mode=buck_mode,
         no_saved_state=no_saved_state,
         search_path=list(search_path),
@@ -521,10 +453,11 @@ def pyre(
         shared_memory_dependency_table_power=shared_memory_dependency_table_power,
         shared_memory_hash_table_power=shared_memory_hash_table_power,
         number_of_workers=number_of_workers,
+        enable_hover=enable_hover,
     )
     if arguments.version:
         _show_pyre_version(arguments)
-        return ExitCode.SUCCESS
+        return commands.ExitCode.SUCCESS
 
     context.ensure_object(dict)
     context.obj["arguments"] = arguments
@@ -533,7 +466,7 @@ def pyre(
         return _run_default_command(arguments)
 
     # This return value is not used anywhere.
-    return ExitCode.SUCCESS
+    return commands.ExitCode.SUCCESS
 
 
 @pyre.command()
@@ -623,65 +556,33 @@ def analyze(
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-
-    if configuration.use_command_v2:
-        _check_configuration(configuration)
-        _start_logging_to_directory(configuration.log_directory)
-        return v2.analyze.run(
-            configuration,
-            command_arguments.AnalyzeArguments(
-                debug=command_argument.debug,
-                dump_call_graph=dump_call_graph,
-                dump_model_query_results=dump_model_query_results,
-                enable_memory_profiling=command_argument.enable_memory_profiling,
-                enable_profiling=command_argument.enable_profiling,
-                find_missing_flows=command_arguments.MissingFlowsKind(
-                    find_missing_flows
-                )
-                if find_missing_flows is not None
-                else None,
-                inline_decorators=inline_decorators,
-                log_identifier=command_argument.log_identifier,
-                maximum_tito_depth=maximum_tito_depth,
-                maximum_trace_length=maximum_trace_length,
-                no_verify=no_verify,
-                output=command_argument.output,
-                repository_root=repository_root,
-                rule=list(rule),
-                save_results_to=save_results_to,
-                sequential=command_argument.sequential,
-                taint_models_path=list(taint_models_path),
-                use_cache=use_cache,
-            ),
-        )
-    else:
-        rules = list(rule)
-        return run_pyre_command(
-            commands.Analyze(
-                command_argument,
-                original_directory=os.getcwd(),
-                configuration=configuration,
-                analysis=analysis,
-                taint_models_path=list(taint_models_path),
-                no_verify=no_verify,
-                save_results_to=save_results_to,
-                dump_call_graph=dump_call_graph,
-                repository_root=repository_root,
-                rules=list(rules) if len(rules) > 0 else None,
-                find_missing_flows=(
-                    command_arguments.MissingFlowsKind(find_missing_flows)
-                    if find_missing_flows is not None
-                    else None
-                ),
-                dump_model_query_results=dump_model_query_results,
-                use_cache=use_cache,
-                inline_decorators=inline_decorators,
-                maximum_trace_length=maximum_trace_length,
-                maximum_tito_depth=maximum_tito_depth,
-            ),
-            configuration,
-            command_argument.noninteractive,
-        )
+    _check_configuration(configuration)
+    _start_logging_to_directory(configuration.log_directory)
+    return commands.analyze.run(
+        configuration,
+        command_arguments.AnalyzeArguments(
+            debug=command_argument.debug,
+            dump_call_graph=dump_call_graph,
+            dump_model_query_results=dump_model_query_results,
+            enable_memory_profiling=command_argument.enable_memory_profiling,
+            enable_profiling=command_argument.enable_profiling,
+            find_missing_flows=command_arguments.MissingFlowsKind(find_missing_flows)
+            if find_missing_flows is not None
+            else None,
+            inline_decorators=inline_decorators,
+            log_identifier=command_argument.log_identifier,
+            maximum_tito_depth=maximum_tito_depth,
+            maximum_trace_length=maximum_trace_length,
+            no_verify=no_verify,
+            output=command_argument.output,
+            repository_root=repository_root,
+            rule=list(rule),
+            save_results_to=save_results_to,
+            sequential=command_argument.sequential,
+            taint_models_path=list(taint_models_path),
+            use_cache=use_cache,
+        ),
+    )
 
 
 @pyre.command()
@@ -819,6 +720,15 @@ def incremental(
         "code."
     ),
 )
+@click.option(
+    "--simple-annotations",
+    is_flag=True,
+    default=False,
+    help=(
+        "Only infer the simplest annotations that are guaranteed to "
+        "codemod with --in-place cleanly."
+    ),
+)
 @click.option(
     "--dequalify",
     is_flag=True,
@@ -843,6 +753,7 @@ def infer(
     annotate_attributes: bool,
     use_future_annotations: bool,
     quote_annotations: bool,
+    simple_annotations: bool,
     dequalify: bool,
 ) -> int:
     """
@@ -862,7 +773,7 @@ def infer(
         if len(paths_to_modify) == 0
         else {working_directory / Path(path) for path in paths_to_modify}
     )
-    return v2.infer.run(
+    return commands.infer.run(
         configuration,
         command_arguments.InferArguments(
             working_directory=working_directory,
@@ -872,6 +783,7 @@ def infer(
             enable_profiling=command_argument.enable_profiling,
             debug_infer=debug_infer,
             quote_annotations=quote_annotations,
+            simple_annotations=simple_annotations,
             dequalify=dequalify,
             log_identifier=command_argument.log_identifier,
             logging_sections=command_argument.logging_sections,
@@ -897,7 +809,7 @@ def init(context: click.Context, local: bool) -> int:
     """
     Create a pyre configuration file at the current directory.
     """
-    return v2.initialize.run()
+    return commands.initialize.run()
 
 
 @pyre.command()
@@ -906,9 +818,9 @@ def init_pysa(context: click.Context) -> int:
     """
     Creates a suitable environment for running pyre analyze.
     """
-    create_configuration_return_code: int = v2.initialize.run()
-    if create_configuration_return_code == ExitCode.SUCCESS:
-        return v2.initialize_pysa.run()
+    create_configuration_return_code: int = commands.initialize.run()
+    if create_configuration_return_code == commands.ExitCode.SUCCESS:
+        return commands.initialize_pysa.run()
     else:
         return create_configuration_return_code
 
@@ -926,7 +838,7 @@ def kill(context: click.Context, with_fire: bool) -> int:
     configuration = configuration_module.create_configuration(
         command_argument, Path(".")
     )
-    return v2.kill.run(configuration, with_fire)
+    return commands.kill.run(configuration, with_fire)
 
 
 @pyre.command()
@@ -945,10 +857,10 @@ def persistent(context: click.Context) -> int:
     _start_logging_to_directory(
         configuration.log_directory,
     )
-    return v2.persistent.run(
+    return commands.persistent.run(
         command_argument,
         base_directory,
-        v2.backend_arguments.RemoteLogging.create(
+        commands.backend_arguments.RemoteLogging.create(
             configuration.logger, command_argument.log_identifier
         ),
     )
@@ -970,7 +882,7 @@ def pysa_language_server(context: click.Context, no_watchman: bool) -> int:
     _start_logging_to_directory(
         configuration.log_directory,
     )
-    return v2.pysa_server.run(
+    return commands.pysa_server.run(
         configuration,
         command_arguments.StartArguments(
             changed_files_path=command_argument.changed_files_path,
@@ -1015,7 +927,7 @@ def profile(context: click.Context, profile_output: str) -> int:
 
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    return v2.profile.run(configuration, get_profile_output(profile_output))
+    return commands.profile.run(configuration, get_profile_output(profile_output))
 
 
 @pyre.command()
@@ -1033,7 +945,7 @@ def query(context: click.Context, query: str) -> int:
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
     _start_logging_to_directory(configuration.log_directory)
-    return v2.query.run(configuration, query)
+    return commands.query.run(configuration, query)
 
 
 @pyre.command()
@@ -1062,7 +974,7 @@ def rage(
     configuration = configuration_module.create_configuration(
         command_argument, Path(".")
     )
-    return v2.rage.run(
+    return commands.rage.run(
         configuration,
         command_arguments.RageArguments(
             output=Path(output_file) if output_file is not None else None,
@@ -1132,7 +1044,7 @@ def restart(
         terminal=terminal,
         wait_on_initialization=True,
     )
-    return v2.restart.run(
+    return commands.restart.run(
         configuration,
         command_arguments.IncrementalArguments(
             output=command_argument.output,
@@ -1152,9 +1064,9 @@ def servers(context: click.Context) -> int:
     """
     if context.invoked_subcommand is None:
         arguments: command_arguments.CommandArguments = context.obj["arguments"]
-        return v2.servers.run_list(arguments.output)
+        return commands.servers.run_list(arguments.output)
     # This return value is not used anywhere.
-    return ExitCode.SUCCESS
+    return commands.ExitCode.SUCCESS
 
 
 @servers.command(name="list")
@@ -1164,7 +1076,7 @@ def servers_list(context: click.Context) -> int:
     List all running servers.
     """
     arguments: command_arguments.CommandArguments = context.obj["arguments"]
-    return v2.servers.run_list(arguments.output)
+    return commands.servers.run_list(arguments.output)
 
 
 @servers.command(name="stop")
@@ -1173,7 +1085,7 @@ def servers_stop(context: click.Context) -> int:
     """
     Stop all running servers.
     """
-    return v2.servers.run_stop()
+    return commands.servers.run_stop()
 
 
 @pyre.command()
@@ -1225,7 +1137,7 @@ def start(
     configuration = _create_configuration_with_retry(command_argument, Path("."))
     _check_configuration(configuration)
     _start_logging_to_directory(configuration.log_directory)
-    return v2.start.run(
+    return commands.start.run(
         configuration,
         command_arguments.StartArguments(
             changed_files_path=command_argument.changed_files_path,
@@ -1275,7 +1187,7 @@ def statistics(
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    return v2.statistics.run(
+    return commands.statistics.run(
         configuration,
         command_arguments.StatisticsArguments(
             filter_paths=list(filter_paths),
@@ -1311,7 +1223,7 @@ def coverage(
     """
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
-    return v2.coverage.run(configuration, working_directory, list(roots))
+    return commands.coverage.run(configuration, working_directory, list(roots))
 
 
 @pyre.command()
@@ -1325,7 +1237,7 @@ def stop(context: click.Context) -> int:
         command_argument, Path(".")
     )
     _start_logging_to_directory(configuration.log_directory)
-    return v2.stop.run(configuration)
+    return commands.stop.run(configuration)
 
 
 @pyre.command()
@@ -1337,7 +1249,7 @@ def validate_models(context: click.Context) -> int:
     command_argument: command_arguments.CommandArguments = context.obj["arguments"]
     configuration = _create_configuration_with_retry(command_argument, Path("."))
     _start_logging_to_directory(configuration.log_directory)
-    return v2.validate_models.run(configuration, output=command_argument.output)
+    return commands.validate_models.run(configuration, output=command_argument.output)
 
 
 # Need the default argument here since this is our entry point in setup.py
@@ -1348,16 +1260,17 @@ def main(argv: List[str] = sys.argv[1:]) -> int:
             return_code = pyre(argv, auto_envvar_prefix="PYRE", standalone_mode=False)
         except configuration_module.InvalidConfiguration as error:
             LOG.error(str(error))
-            return ExitCode.CONFIGURATION_ERROR
+            return commands.ExitCode.CONFIGURATION_ERROR
         except click.ClickException as error:
             error.show()
-            return_code = ExitCode.FAILURE
+            return_code = commands.ExitCode.FAILURE
         except commands.ClientException as error:
-            LOG.error(str(error))
+            for line in str(error).split("\n"):
+                LOG.error(line)
             return_code = error.exit_code
         except Exception as error:
             LOG.error(str(error))
-            return_code = ExitCode.FAILURE
+            return_code = commands.ExitCode.FAILURE
     return return_code
 
 
@@ -1369,5 +1282,5 @@ if __name__ == "__main__":
             "Pyre could not determine the current working directory. "
             "Has it been removed?\nExiting."
         )
-        sys.exit(ExitCode.FAILURE)
+        sys.exit(commands.ExitCode.FAILURE)
     sys.exit(main(sys.argv[1:]))
